@@ -3,7 +3,7 @@
  *
  * Author: H.G.Muller (August 2009)
  *
- * Copyright 2009, 2010 Free Software Foundation, Inc.
+ * Copyright 2009, 2010, 2011 Free Software Foundation, Inc.
  *
  * ------------------------------------------------------------------------
  *
@@ -32,6 +32,7 @@
 #include <malloc.h>
 #include <commdlg.h>
 #include <dlgs.h>
+#include <Windowsx.h>
 
 #include "common.h"
 #include "frontend.h"
@@ -41,24 +42,30 @@
 #include "wsnap.h"
 
 int chatCount;
+static int onTop;
 extern char chatPartner[MAX_CHAT][MSG_SIZ];
 HANDLE chatHandle[MAX_CHAT];
+static WNDPROC chatInputWindowProc;
 
 void SendToICS P((char *s));
-void ChatPopUp();
+void ChatPopUp P((char *s));
 void ChatPopDown();
 
 /* Imports from backend.c */
-char * SavePart(char *str);
 extern int opponentKibitzes;
 
 /* Imports from winboard.c */
+VOID SaveInHistory(char *cmd);
+char *PrevInHistory(char *cmd);
+char *NextInHistory();
 extern HWND ChatDialog;
 
 extern HINSTANCE hInst;
-extern HWND hwndMain;
+extern HWND hwndConsole;
+extern char ics_handle[];
 
 extern WindowPlacement wpChat[MAX_CHAT];
+extern WindowPlacement wpConsole;
 
 extern BoardSize boardSize;
 
@@ -133,31 +140,109 @@ static void InsertIntoMemo( HANDLE hDlg, char * text )
     SendMessage( hMemo, EM_SCROLLCARET, 0, 0);
 }
 
+LRESULT CALLBACK
+InterceptArrowKeys(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
+{
+  char buf[MSG_SIZ];
+  char *p;
+  CHARRANGE sel;
+
+  switch (message) {
+  case WM_KEYDOWN: // cloned from ConsoleInputSubClass()
+    switch (wParam) {
+    case VK_UP:
+      GetWindowText(hwnd, buf, MSG_SIZ);
+      p = PrevInHistory(buf);
+      if (p != NULL) {
+	SetWindowText(hwnd, p);
+	sel.cpMin = 999999;
+	sel.cpMax = 999999;
+	SendMessage(hwnd, EM_EXSETSEL, 0, (LPARAM)&sel);
+        return 0;
+      }
+      break;
+    case VK_DOWN:
+      p = NextInHistory();
+      if (p != NULL) {
+	SetWindowText(hwnd, p);
+	sel.cpMin = 999999;
+	sel.cpMax = 999999;
+	SendMessage(hwnd, EM_EXSETSEL, 0, (LPARAM)&sel);
+        return 0;
+      }
+      break;
+    }
+  }
+  return (*chatInputWindowProc)(hwnd, message, wParam, lParam);
+}
+
 // This seems pure front end
 LRESULT CALLBACK ChatProc( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam )
 {
     static SnapData sd;
     char buf[MSG_SIZ], mess[MSG_SIZ];
-    int partner = -1, i;
+    int partner = -1, i, x, y;
     static BOOL filterHasFocus[MAX_CHAT];
+    WORD wMask;
+    HWND hMemo;
 
     for(i=0; i<MAX_CHAT; i++) if(hDlg == chatHandle[i]) { partner = i; break; }
 
     switch (message) {
     case WM_INITDIALOG:
+        Translate(hDlg, DLG_Chat);
 	if(partner<0) {
 		for(i=0; i<MAX_CHAT; i++) if(chatHandle[i] == NULL) { partner = i; break; }
 	        chatHandle[partner] = hDlg;
-		sprintf(buf, "Chat Window %s", first.tidy);
+		snprintf(buf, MSG_SIZ, T_("Chat Window %s"), ics_handle[0] ? ics_handle : first.tidy);
 		SetWindowText(hDlg, buf);
         }
-	chatPartner[partner][0] = 0;
-	filterHasFocus[partner] = FALSE;
-
+	for(i=0; i<MAX_CHAT; i++) if(chatHandle[i]) {
+	    if(i == partner) continue;
+	    // set our button in other open chats
+	    SetDlgItemText(chatHandle[i], IDC_Focus1+partner-(i<partner), chatPartner[partner]);
+	    EnableWindow( GetDlgItem(chatHandle[i], IDC_Focus1+partner-(i<partner)), 1 );
+	    // and buttons for other chats in ours
+	    SetDlgItemText(hDlg, IDC_Focus1+i-(i>partner), chatPartner[i]);
+	} else EnableWindow( GetDlgItem(hDlg, IDC_Focus1+i-(i>partner)), 0 );
+	for(i=0; i<MAX_CHAT-1; i++) { Button_SetStyle(GetDlgItem(hDlg, IDC_Focus1+i), BS_PUSHBUTTON|BS_LEFT, TRUE); }
+        x = wpConsole.x; y = wpConsole.y; EnsureOnScreen(&x, &y, 0, 0);
+        SetWindowPos(hDlg, NULL, x, y, 0, 0, SWP_NOZORDER|SWP_NOSIZE);
+	SendMessage( GetDlgItem(hDlg, IDC_ChatPartner), // [HGM] clickbox: initialize with requested handle
+			WM_SETTEXT, 0, (LPARAM) chatPartner[partner] );
+	filterHasFocus[partner] = TRUE;
+	onTop = partner; // a newly opened box becomes top one
+	if(chatPartner[partner][0]) {
+	    filterHasFocus[partner] = FALSE;
+	    SetFocus( GetDlgItem(hDlg, OPT_ChatInput) );
+	}
+	hMemo = GetDlgItem(hDlg, IDC_ChatMemo);
+	wMask = (WORD) SendMessage(hMemo, EM_GETEVENTMASK, 0, 0L);
+	SendMessage(hMemo, EM_SETEVENTMASK, 0, wMask | ENM_LINK);
+	SendMessage(hMemo, EM_AUTOURLDETECT, TRUE, 0L);
+	chatInputWindowProc = (WNDPROC) // cloned from ConsoleWndProc(). Assume they all share same proc.
+	      SetWindowLong(GetDlgItem(hDlg, OPT_ChatInput), GWL_WNDPROC, (LONG) InterceptArrowKeys);
         return FALSE;
 
+    case WM_NOTIFY:
+      if (((NMHDR*)lParam)->code == EN_LINK)
+      {
+	ENLINK *pLink = (ENLINK*)lParam;
+	if (pLink->msg == WM_LBUTTONUP)
+	{
+	  TEXTRANGE tr;
+
+	  tr.chrg = pLink->chrg;
+	  tr.lpstrText = malloc(1+tr.chrg.cpMax-tr.chrg.cpMin);
+	  SendMessage( GetDlgItem(hDlg, IDC_ChatMemo), EM_GETTEXTRANGE, 0, (LPARAM)&tr);
+	  ShellExecute(NULL, "open", tr.lpstrText, NULL, NULL, SW_SHOW);
+	  free(tr.lpstrText);
+	}
+      }
+    break;
+
     case WM_COMMAND:
-      /* 
+      /*
         [AS]
         If <Enter> is pressed while editing the filter, it's better to apply
         the filter rather than selecting the current game.
@@ -181,12 +266,9 @@ LRESULT CALLBACK ChatProc( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam
 
         switch (LOWORD(wParam)) {
 
-	case IDCANCEL:
-	    chatHandle[partner] = 0;
-	    chatPartner[partner][0] = 0;
-            ChatPopDown();
-	    EndDialog(hDlg, TRUE);
-            break;
+	case IDCANCEL: /* let Esc key switch focus back to console */
+	    SetFocus(GetDlgItem(hwndConsole, OPT_ConsoleInput));
+	    break;
 
 	case IDC_Clear:
 	    SendMessage( GetDlgItem(hDlg, IDC_ChatMemo), WM_SETTEXT, 0, (LPARAM) "" );
@@ -194,25 +276,48 @@ LRESULT CALLBACK ChatProc( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam
 
 	case IDC_Change:
 	    GetDlgItemText(hDlg, IDC_ChatPartner, chatPartner[partner], MSG_SIZ);
+	    for(i=0; i<MAX_CHAT; i++) if(chatHandle[i] && i != partner) {
+	      // set our button in other open chats
+	      SetDlgItemText(chatHandle[i], IDC_Focus1+partner-(i<partner), chatPartner[partner]);
+	    }
 	    break;
 
 	case IDC_Send:
 	    GetDlgItemText(hDlg, OPT_ChatInput, mess, MSG_SIZ);
 	    SetDlgItemText(hDlg, OPT_ChatInput, "");
 	    // from here on it could be back-end
-	    if(!strcmp("WHISPER", chatPartner[partner]))
-		sprintf(buf, "whisper %s\n", mess); // WHISPER box uses "whisper" to send
+	    SaveInHistory(mess);
+	    if(!strcmp("whispers", chatPartner[partner]))
+	      snprintf(buf, MSG_SIZ, "whisper %s\n", mess); // WHISPER box uses "whisper" to send
+	    else if(!strcmp("shouts", chatPartner[partner]))
+	      snprintf(buf, MSG_SIZ, "shout %s\n", mess); // SHOUT box uses "shout" to send
 	    else {
 		if(!atoi(chatPartner[partner])) {
-		    sprintf(buf, "> %s\r\n", mess); // echo only tells to handle, not channel
+		  snprintf(buf, MSG_SIZ, "> %s\r\n", mess); // echo only tells to handle, not channel
 		InsertIntoMemo(hDlg, buf);
-		sprintf(buf, "xtell %s %s\n", chatPartner[partner], mess);
+		snprintf(buf, MSG_SIZ, "xtell %s %s\n", chatPartner[partner], mess);
 		} else
-		sprintf(buf, "tell %s %s\n", chatPartner[partner], mess);
+		  snprintf(buf, MSG_SIZ, "tell %s %s\n", chatPartner[partner], mess);
 	    }
 	    SendToICS(buf);
 	    break;
 
+	case IDC_Focus1:
+	case IDC_Focus2:
+	case IDC_Focus3:
+	case IDC_Focus4:
+	    i = LOWORD(wParam) - IDC_Focus1;
+	    if(i >= partner) i++;
+	    onTop = i;
+	    SetFocus(GetDlgItem(hDlg, IDC_Send));
+	    if(chatHandle[i]) {
+		int j;
+		for(j=0; j<MAX_CHAT; j++) if(i != j && chatHandle[j])
+		    Button_SetState(GetDlgItem(chatHandle[j], IDC_Focus1+i-(j<i)), FALSE);
+		SetFocus(GetDlgItem(chatHandle[i], OPT_ChatInput));
+	    }
+	    break;
+
         default:
           break;
         }
@@ -223,6 +328,11 @@ LRESULT CALLBACK ChatProc( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam
 	chatHandle[partner] = 0;
 	chatPartner[partner][0] = 0;
         ChatPopDown();
+	for(i=0; i<MAX_CHAT; i++) if(chatHandle[i] && i != partner) {
+	    // set our button in other open chats
+	    SetDlgItemText(chatHandle[i], IDC_Focus1+partner-(i<partner), "");
+	    EnableWindow( GetDlgItem(chatHandle[i], IDC_Focus1+partner-(i<partner)), 0 );
+	}
 	EndDialog(hDlg, TRUE);
         break;
 
@@ -247,19 +357,24 @@ LRESULT CALLBACK ChatProc( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam
 }
 
 // front end
-void ChatPopUp()
+void ChatPopUp(char *icsHandle)
 {
   FARPROC lpProc;
-  
-  if(chatCount >= MAX_CHAT) return;
+  int i, partner = -1;
 
   CheckMenuItem(GetMenu(hwndMain), IDM_NewChat, MF_CHECKED);
+  for(i=0; i<MAX_CHAT; i++) if(chatHandle[i] == NULL) { partner = i; break; }
+  if(partner == -1) { DisplayError("You first have to close a Chat Box\nbefore you can open a new one", 0); return; }
+  if(icsHandle) // [HGM] clickbox set handle in advance
+    safeStrCpy(chatPartner[partner], icsHandle,
+	       sizeof(chatPartner[partner])/sizeof(chatPartner[partner][0]) );
+  else chatPartner[partner][0] = NULLCHAR;
   chatCount++;
 
     lpProc = MakeProcInstance( (FARPROC) ChatProc, hInst );
 
     /* Note to self: dialog must have the WS_VISIBLE style set, otherwise it's not shown! */
-    CreateDialog( hInst, MAKEINTRESOURCE(DLG_Chat), hwndMain, (DLGPROC)lpProc );
+    CreateDialog( hInst, MAKEINTRESOURCE(DLG_Chat), hwndConsole, (DLGPROC)lpProc );
 
     FreeProcInstance(lpProc);
 
@@ -277,9 +392,11 @@ void ChatPopDown()
 
 void OutputChatMessage(int partner, char *text)
 {
-	if(!chatHandle[partner]) return;
+	int j, n = strlen(text);
 
-	int n = strlen(text);
+	if(!chatHandle[partner]) return;
 	text[n+1] = 0; text[n] = '\n'; text[n-1] = '\r'; // Needs CR to not lose line breaks on copy-paste
 	InsertIntoMemo(chatHandle[partner], text);
+	if(partner != onTop) for(j=0; j<MAX_CHAT; j++) if(j != partner && chatHandle[j])
+	    Button_SetState(GetDlgItem(chatHandle[j], IDC_Focus1+partner-(j<partner)), TRUE);
 }
