@@ -1,7 +1,7 @@
 /*
  * Engine-settings dialog. The complexity come from an attempt to present the engine-defined options
  * in a nicey formatted layout. To this end we first run a back-end pre-formatter, which will distribute
- * the controls over two columns (the minimum required, as some are double width). It also takes care of 
+ * the controls over two columns (the minimum required, as some are double width). It also takes care of
  * grouping options that start with the same word (mainly for "Polyglot ..." options). It assigns relative
  * suitability to break points between lines, and in the end decides if and where to break up the list
  * for display in multiple (2*N) columns.
@@ -15,9 +15,13 @@
 #include <stdio.h>
 #include <string.h>
 #include "common.h"
+#include "frontend.h"
 #include "backend.h"
+#include "winboard.h"
 #include "backendz.h"
 
+#define _(s) T_(s)
+
 int layoutList[2*MAX_OPTIONS];
 int checkList[2*MAX_OPTIONS];
 int comboList[2*MAX_OPTIONS];
@@ -100,7 +104,7 @@ LayoutOptions(int firstOption, int endOption, char *groupName, Option *optionLis
 	    nextOption++;
 	}
 	// We now must be at the end, or looking at a spin or textbox (in nextType)
-	if(!stop) 
+	if(!stop)
 	    nextType = Button; // kudge to flush remaining checks and combos undistorted
 	// Take a new line if a spin follows combos or checks, or when we encounter a textbox
 	if((combos+checks || nextType == TextBox) && layout&1) {
@@ -118,7 +122,7 @@ LayoutOptions(int firstOption, int endOption, char *groupName, Option *optionLis
 	if(nextType == TextBox && lastType == ComboBox)
 	    checkList[checks++] = comboList[--combos];
 	// Now append the checks behind the (remaining) combos to treat them as one group
-	for(i=0; i< checks; i++) 
+	for(i=0; i< checks; i++)
 	    comboList[combos++] = checkList[i];
 	// emit the consecutive checks and combos in two columns
 	right = combos/2; // rounded down if odd!
@@ -185,7 +189,7 @@ DesignOptionDialog(ChessProgramState *cps)
 		    if( cps->option[k].name[j] != cps->option[k+groupSize].name[j]) break;
 		groupNameLength = j;
 		groupSize++;
-		
+
 	}
 	if(groupSize > 3) {
 		// We found a group to terminates the current section
@@ -199,7 +203,7 @@ DesignOptionDialog(ChessProgramState *cps)
     }
     if(n != k) LayoutOptions(n, k, "", cps->option); // flush remaining solitary options
     // decide if and where we break into two column pairs
-    
+
     // Emit buttons and add OK and cancel
 //    for(k=0; k<buttons; k++) layoutList[layout++] = buttonList[k];
     // Create the dialog window
@@ -275,12 +279,12 @@ SetOptionValues(HWND hDlg, ChessProgramState *cps)
 		break;
 	}
     }
-    SetDlgItemText( hDlg, IDOK, "OK" );
-    SetDlgItemText( hDlg, IDCANCEL, "Cancel" );
-    sprintf(title, "%s Engine Settings (%s)", cps->which, cps->tidy); 
+    SetDlgItemText( hDlg, IDOK, _("OK") );
+    SetDlgItemText( hDlg, IDCANCEL, _("Cancel") );
+    snprintf(title, MSG_SIZ, _("%s Engine Settings (%s)"), T_(cps->which), cps->tidy);
     title[0] &= ~32; // capitalize
     SetWindowText( hDlg, title);
-    for(i=0; i<groups; i+=2) { 
+    for(i=0; i<groups; i+=2) {
 	int id, p; char buf[MSG_SIZ];
 	id = k = boxList[i];
 	if(layoutList[k] < 0) k++;
@@ -319,7 +323,7 @@ GetOptionValues(HWND hDlg, ChessProgramState *cps)
 		success = GetDlgItemText( hDlg, 2001+2*i, newText, MSG_SIZ - strlen(cps->option[j].name) - 9 );
 		if(!success) break;
 		changed = strcmp(cps->option[j].textValue, newText) != 0;
-		strcpy(cps->option[j].textValue, newText);
+		safeStrCpy(cps->option[j].textValue, newText, MSG_SIZ - (cps->option[j].textValue - cps->option[j].name) );
 		break;
 	    case CheckBox:
 		new = IsDlgButtonChecked( hDlg, 2000+2*i );
@@ -342,8 +346,10 @@ GetOptionValues(HWND hDlg, ChessProgramState *cps)
 	    default:
 		break; // are treated instantly, so they have been sent already
 	}
-	if(changed == 2) sprintf(buf, "option %s=%d\n", cps->option[j].name, new); else
-	if(changed == 1) sprintf(buf, "option %s=%s\n", cps->option[j].name, newText);
+	if(changed == 2)
+	  snprintf(buf, MSG_SIZ, "option %s=%d\n", cps->option[j].name, new); else
+	if(changed == 1)
+	  snprintf(buf, MSG_SIZ, "option %s=%s\n", cps->option[j].name, newText);
 	if(changed) SendToProgram(buf, cps);
     }
 }
@@ -372,7 +378,7 @@ LRESULT CALLBACK SettingsProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lPa
             return TRUE;
 
         case IDCANCEL:
-            EndDialog( hDlg, 1 );   
+            EndDialog( hDlg, 1 );
             return TRUE;
 
 	default:
@@ -381,10 +387,10 @@ LRESULT CALLBACK SettingsProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lPa
 	    if( i>=2000 &&  i < 2000+2*(layout+buttons)) {
 		j = layoutList[(i - 2000)/2];
 		if(j == -2) {
-		          char filter[] = 
+		          char filter[] =
 				"All files\0*.*\0BIN Files\0*.bin\0LOG Files\0*.log\0INI Files\0*.ini\0\0";
 /*
-{ 
+{
 		              'A','l','l',' ','F','i','l','e','s', 0,
 		              '*','.','*', 0,
 		              'B','I','N',' ','F','i','l','e','s', 0,
@@ -393,7 +399,7 @@ LRESULT CALLBACK SettingsProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lPa
 */
 		          OPENFILENAME ofn;
 
-		          strcpy( buf, "" );
+		          safeStrCpy( buf, "" , sizeof( buf)/sizeof( buf[0]) );
 
 		          ZeroMemory( &ofn, sizeof(ofn) );
 
@@ -403,7 +409,7 @@ LRESULT CALLBACK SettingsProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lPa
 		          ofn.lpstrFilter = filter;
 		          ofn.lpstrFile = buf;
 		          ofn.nMaxFile = sizeof(buf);
-		          ofn.lpstrTitle = "Choose Book";
+		          ofn.lpstrTitle = _("Choose Book");
 		          ofn.Flags = OFN_FILEMUSTEXIST | OFN_LONGNAMES | OFN_HIDEREADONLY;
 
 		          if( GetOpenFileName( &ofn ) ) {
@@ -413,8 +419,11 @@ LRESULT CALLBACK SettingsProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lPa
 		if(j < 0) break;
 		if( activeCps->option[j].type  == SaveButton)
 		     GetOptionValues(hDlg, activeCps);
-		else if( activeCps->option[j].type  != Button) break;
-		sprintf(buf, "option %s\n", activeCps->option[j].name);
+		else if( activeCps->option[j].type  == ResetButton) {
+		     ClearOptions(activeCps);
+		     EndDialog( hDlg, 0 );
+		} else if( activeCps->option[j].type  != Button) break;
+		snprintf(buf, MSG_SIZ, "option %s\n", activeCps->option[j].name);
 		SendToProgram(buf, activeCps);
 	    }
 	    break;
@@ -480,7 +489,7 @@ void AddOption(int x, int y, Control type, int i)
 	case Message:
 	    break;
     }
-    
+
 }
 
 void
@@ -491,7 +500,7 @@ CreateDialogTemplate(int *layoutList, int nr, ChessProgramState *cps)
     template.header.cdit = 0;
     template.header.cx = 307;
     buttonRows = (buttons + 1 + 3)/4; // 4 per row, rounded up
-    if(nr > 50) { 
+    if(nr > 50) {
 	breakPoint = (nr+2*buttonRows+1)/2 & ~1;
 	template.header.cx = 625;
     }
@@ -499,9 +508,9 @@ CreateDialogTemplate(int *layoutList, int nr, ChessProgramState *cps)
     for(i=0; i<nr; i++) {
 	if(k < groups && i == boxList[k]) {
 	    y += 10;
-	    AddControl(x+2, y+13*(i>>1)-2, 301, 13*(boxList[k+1]-boxList[k]>>1)+8, 
+	    AddControl(x+2, y+13*(i>>1)-2, 301, 13*(boxList[k+1]-boxList[k]>>1)+8,
 						0x0082, WS_VISIBLE | WS_CHILD | SS_BLACKFRAME, 2400);
-	    AddControl(x+60, y+13*(i>>1)-6, 10*groupNameList[k]/3, 10, 
+	    AddControl(x+60, y+13*(i>>1)-6, 10*groupNameList[k]/3, 10,
 						0x0082, SS_ENDELLIPSIS | WS_VISIBLE | WS_CHILD, 2*(i+MAX_OPTIONS));
 	}
 	j = layoutList[i];
@@ -515,8 +524,8 @@ CreateDialogTemplate(int *layoutList, int nr, ChessProgramState *cps)
     // add butons at the bottom of dialog window
     y += 13*(nr>>1)+5;
 
-    AddControl(x+275, y+18*(buttonRows-1), 25, 15, 0x0080, BS_PUSHBUTTON | WS_VISIBLE | WS_CHILD, IDOK-2000);
-    AddControl(x+235, y+18*(buttonRows-1), 35, 15, 0x0080, BS_PUSHBUTTON | WS_VISIBLE | WS_CHILD, IDCANCEL-2000);
+    AddControl(x+225, y+18*(buttonRows-1), 30, 15, 0x0080, BS_PUSHBUTTON | WS_VISIBLE | WS_CHILD, IDOK-2000);
+    AddControl(x+260, y+18*(buttonRows-1), 40, 15, 0x0080, BS_PUSHBUTTON | WS_VISIBLE | WS_CHILD, IDCANCEL-2000);
     for(i=0; i<buttons; i++) {
 	AddControl(x+70*(i%4)+5, y+18*(i/4), 65, 15, 0x0080, BS_PUSHBUTTON | WS_VISIBLE | WS_CHILD, 2*(nr+i));
 	layoutList[nr+i] = buttonList[i];
@@ -526,7 +535,7 @@ CreateDialogTemplate(int *layoutList, int nr, ChessProgramState *cps)
     template.header.style &= ~WS_VSCROLL;
 }
 
-void 
+void
 EngineOptionsPopup(HWND hwnd, ChessProgramState *cps)
 {
     FARPROC lpProc = MakeProcInstance( (FARPROC) SettingsProc, hInst );
